@@ -15,10 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-#
-#
-# Modified by Yusuf Kaka <yusufk at gmail>
-# Added support for Optional TLV's
 
 """SMPP client module"""
 
@@ -27,6 +23,7 @@ import logging
 import select
 import socket
 import struct
+import warnings
 
 from smpplib import consts, exceptions, smpp
 
@@ -60,18 +57,43 @@ class Client(object):
     port = None
     vendor = None
     _socket = None
+    _ssl_context = None
     sequence_generator = None
 
-    def __init__(self, host, port, timeout=5, sequence_generator=None, logger_name=None):
+    def __init__(
+        self,
+        host,
+        port,
+        timeout=5,
+        sequence_generator=None,
+        logger_name=None,
+        ssl_context=None,
+        allow_unknown_opt_params=None,
+    ):
         self.host = host
         self.port = int(port)
-        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self._ssl_context = ssl_context
         self.timeout = timeout
         self.logger = logging.getLogger(logger_name or 'smpp.Client.{}'.format(id(self)))
         if sequence_generator is None:
             sequence_generator = SimpleSequenceGenerator()
         self.sequence_generator = sequence_generator
 
+        if allow_unknown_opt_params is None:
+            warnings.warn(
+                "Unknown optional parameters during PDU parsing will stop "
+                "causing an exception in a future smpplib version "
+                "(in order to comply with the SMPP spec). To switch behavior "
+                "now set allow_unknown_opt_params to True.",
+                DeprecationWarning,
+            )
+            self.allow_unknown_opt_params = False
+        else:
+            self.allow_unknown_opt_params = allow_unknown_opt_params
+
+
+        self._socket = self._create_socket()
+
     def __enter__(self):
         return self
 
@@ -97,6 +119,15 @@ class Client(object):
     def next_sequence(self):
         return self.sequence_generator.next_sequence()
 
+    def _create_socket(self):
+        raw_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        raw_socket.settimeout(self.timeout)
+
+        if self._ssl_context is None:
+            return raw_socket
+
+        return self._ssl_context.wrap_socket(raw_socket)
+
     def connect(self):
         """Connect to SMSC"""
 
@@ -104,8 +135,7 @@ class Client(object):
 
         try:
             if self._socket is None:
-                self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                self._socket.settimeout(self.timeout)
+                self._socket = self._create_socket()
             self._socket.connect((self.host, self.port))
             self.state = consts.SMPP_CLIENT_STATE_OPEN
         except socket.error:
@@ -219,11 +249,24 @@ class Client(object):
 
         raw_pdu = raw_len
         while len(raw_pdu) < length:
-            raw_pdu += self._socket.recv(length - len(raw_pdu))
+            try:
+                raw_pdu_part = self._socket.recv(length - len(raw_pdu))
+            except socket.timeout:
+                raise
+            except socket.error as e:
+                self.logger.warning(e)
+                raise exceptions.ConnectionError()
+            if not raw_pdu:
+                raise exceptions.ConnectionError()
+            raw_pdu += raw_pdu_part
 
         self.logger.debug('<<%s (%d bytes)', binascii.b2a_hex(raw_pdu), len(raw_pdu))
 
-        pdu = smpp.parse_pdu(raw_pdu, client=self)
+        pdu = smpp.parse_pdu(
+            raw_pdu,
+            client=self,
+            allow_unknown_opt_params=self.allow_unknown_opt_params,
+        )
 
         self.logger.debug('Read %s PDU', pdu.command)
 
@@ -270,6 +313,10 @@ class Client(object):
         """Set new function to handle query resp event"""
         self.query_resp_handler = func
 
+    def set_error_pdu_handler(self, func):
+        """Set new function to handle PDUs with an error status"""
+        self.error_pdu_handler = func
+
     def message_received_handler(self, pdu, **kwargs):
         """Custom handler to process received message. May be overridden"""
         self.logger.warning('Message received handler (Override me)')
@@ -285,8 +332,24 @@ class Client(object):
         """Custom handler to process response to queries. May be overridden"""
         self.logger.warning('Query resp handler (Override me)')
 
+    def error_pdu_handler(self, pdu):
+        raise exceptions.PDUError('({}) {}: {}'.format(
+            pdu.status,
+            pdu.command,
+            consts.DESCRIPTIONS.get(pdu.status, 'Unknown status')),
+            int(pdu.status),
+        )
+
     def read_once(self, ignore_error_codes=None, auto_send_enquire_link=True):
         """Read a PDU and act"""
+
+        if ignore_error_codes is not None:
+            warnings.warn(
+                "ignore_error_codes is deprecated, use set_error_pdu_handler to "
+                "configure a custom error PDU handler instead.",
+                DeprecationWarning,
+            )
+
         try:
             try:
                 pdu = self.read_pdu()
@@ -299,12 +362,7 @@ class Client(object):
                 return
 
             if pdu.is_error():
-                raise exceptions.PDUError('({}) {}: {}'.format(
-                    pdu.status,
-                    pdu.command,
-                    consts.DESCRIPTIONS.get(pdu.status, 'Unknown status')),
-                    int(pdu.status),
-                )
+                self.error_pdu_handler(pdu)
 
             if pdu.command == 'unbind':  # unbind_res
                 self.logger.info('Unbind command received')
