@@ -15,9 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-#
-# Modified by Yusuf Kaka <yusufk at gmail>
-# Added support for Optional TLV's
 
 
 """SMPP Commands module"""
@@ -93,9 +90,11 @@ class Command(pdu.PDU):
 
     params = {}
 
-    def __init__(self, command, need_sequence=True, **kwargs):
+    def __init__(self, command, need_sequence=True, allow_unknown_opt_params=False, **kwargs):
         super(Command, self).__init__(**kwargs)
 
+        self.allow_unknown_opt_params = allow_unknown_opt_params
+
         self.command = command
         if need_sequence and (kwargs.get('sequence') is None):
             self.sequence = self._next_seq()
@@ -171,7 +170,7 @@ class Command(pdu.PDU):
             size = self.params[field].size
             value = field_value.ljust(size, chr(0))
         elif hasattr(self.params[field], 'max'):
-            if len(field_value or '') > self.params[field].max:
+            if len(field_value or '') >= self.params[field].max:
                 field_value = field_value[0:self.params[field].max - 1]
 
             if field_value:
@@ -328,9 +327,17 @@ class Command(pdu.PDU):
 
         while pos < dlen:
             type_code, pos = unpack_short(data, pos)
-            field = get_optional_name(type_code)
             length, pos = unpack_short(data, pos)
 
+            try:
+                field = get_optional_name(type_code)
+            except exceptions.UnknownCommandError as e:
+                if self.allow_unknown_opt_params:
+                    logger.warning("Unknown optional parameter type 0x%x, skipping", type_code)
+                    pos += length
+                    continue
+                raise
+
             param = self.params[field]
             if param.type is int:
                 data, pos = self._parse_int(field, data, pos)
@@ -560,6 +567,8 @@ class DataSMResp(Command):
 class GenericNAck(Command):
     """General Negative Acknowledgement class"""
 
+    params = {}
+    params_order = ()
     _defs = []
 
     def __init__(self, command, **kwargs):
